<?php
// Copyright Lab17 Marek Niemczyk, https://podlewaj.pl, 2020

abstract class PinSetup
{
    const Output = 0;
    const Input = 1;
}

class InvalidPinException extends Exception
{
}

class Gpio
{
    public function getGpio(array $pinNumbers)
    {
        if (!is_array($pinNumbers)){
            throw new Exception("getGpio expects array");
        }

        $result = array();

        foreach ($pinNumbers as $pin){
            if ($this->isValidPinNumber($pin)){
                $status = exec("gpio read ".$pin);
                $result[$pin] = intval($status);
            } else {
                throw new InvalidPinException("Pin $pin is outside wiringPi pin numbers range");
            }
        }

        return $result;
    }

    public function setGpio(int $pin, int $state){
        if ($state != 0 && $state != 1){
            throw new Exception("State should be either 0 or 1");
        }

        if ($this->isValidPinNumber($pin)){
            $this->setupGpio(PinSetup::Output, $pin);
            system("gpio write ".$pin." ".$state);
        } else {
            throw new InvalidPinException("Pin $pin is outside wiringPi pin numbers range");
        }
    }

    private function isValidPinNumber($pinNumber){
        if (
            $pinNumber < 0
            || ($pinNumber > 16 && $pinNumber < 21)
            || $pinNumber > 31
        ) 
        {
            return false;
        }
        return true;
    }

    private function setupGpio(int $inputOutput, int $pinNumber)
    {
        if (!$this->isValidPinNumber($pinNumber)){
            throw new InvalidPinException("Invalid pin $pinNumber - expecting int from wiringPi numeration");
        }

        if ($inputOutput == PinSetup::Input) {
            system("gpio mode " . $pinNumber . " in");
        } else {
            system("gpio mode " . $pinNumber . " out");
        }
    }
}
